
#include "ReadPCX.h"

#include <stdio.h>

unsigned char* ReadPCXMem ( unsigned char* binary_, int size_, unsigned long& width_, unsigned long& height_ )
{
   DWORD resLength = size_ ;
   unsigned char* resData = binary_ ;
   int xMin = resData[4]+(256*resData[5]) ;
   int yMin = resData[6]+(256*resData[7]) ;
   int xMax = resData[8]+(256*resData[9]) ;
   int yMax = resData[10]+(256*resData[11]) ;
   unsigned char* result ;
   unsigned char* inPos ;
   unsigned char* outPos ;
   unsigned int y ;
   int outLineLength ;
   int planes = resData[65] ;
   unsigned char* palette = NULL ;

   if ( planes == 1 )
   {
      palette = resData+resLength-768 ;
   }

   width_ = 1+(xMax-xMin) ;
   height_ = 1+(yMax-yMin) ;

   outLineLength = 4 * width_ ;

   result = new unsigned char [width_*height_*4] ;
   outPos = result ;
   inPos = resData + 128 ;

   for ( y = 0 ; y < height_ ; y++ )
   {
      unsigned char* lineStart ;
      unsigned int x ;

      if ( planes == 1 )
      {
         lineStart = outPos ;
         x = 0 ;
         while ( x < width_ )
         {
            unsigned char byte1 = *inPos++ ;

            if ( ( byte1 & 0xC0 ) == 0xC0 )
            {
               unsigned char byte2 = *inPos++ ;
               unsigned char* palettePos = palette+(3*byte2) ;
               byte1 = byte1 & 0x3F ;
               x += byte1 ;

               while ( byte1-- > 0 )
               {
                  *lineStart++ = 0xFF ;
                  *lineStart++ = palettePos[0] ;
                  *lineStart++ = palettePos[1] ;
                  *lineStart++ = palettePos[2] ;
               }
            }
            else
            {
               unsigned char* palettePos = palette+(3*byte1) ;
               *lineStart++ = 0xFF ;
               *lineStart++ = *palettePos++ ;
               *lineStart++ = *palettePos++ ;
               *lineStart++ = *palettePos ;
               x++ ;
            }
         }
      }
      else
      {
         // Red
         lineStart = outPos+1 ;
         x = 0 ;
         while ( x < width_ )
         {
            unsigned char byte1 = *inPos++ ;

            if ( ( byte1 & 0xC0 ) == 0xC0 )
            {
               unsigned char byte2 = *inPos++ ;
               byte1 = byte1 & 0x3F ;
               x += byte1 ;

               while ( byte1-- > 0 )
               {
                  *lineStart = byte2 ;
                  lineStart += 4 ;
               }
            }
            else
            {
               *lineStart = byte1 ;
               lineStart += 4 ;
               x++ ;
            }
         }


         // Green
         lineStart = outPos+2 ;
         x = 0 ;
         while ( x < width_ )
         {
            unsigned char byte1 = *inPos++ ;

            if ( ( byte1 & 0xC0 ) == 0xC0 )
            {
               unsigned char byte2 = *inPos++ ;
               byte1 = byte1 & 0x3F ;
               x += byte1 ;

               while ( byte1-- > 0 )
               {
                  *lineStart = byte2 ;
                  lineStart += 4 ;
               }
            }
            else
            {
               *lineStart = byte1 ;
               lineStart += 4 ;
               x++ ;
            }
         }


         // Blue
         lineStart = outPos+3 ;
         x = 0 ;
         while ( x < width_ )
         {
            unsigned char byte1 = *inPos++ ;

            if ( ( byte1 & 0xC0 ) == 0xC0 )
            {
               unsigned char byte2 = *inPos++ ;
               byte1 = byte1 & 0x3F ;
               x += byte1 ;

               while ( byte1-- > 0 )
               {
                  *lineStart = byte2 ;
                  lineStart += 4 ;
               }
            }
            else
            {
               *lineStart = byte1 ;
               lineStart += 4 ;
               x++ ;
            }
         }


         if ( planes == 4 )
         {
            // Alpha
            lineStart = outPos ;
            x = 0 ;
            while ( x < width_ )
            {
               unsigned char byte1 = *inPos++ ;

               if ( ( byte1 & 0xC0 ) == 0xC0 )
               {
                  unsigned char byte2 = *inPos++ ;
                  byte1 = byte1 & 0x3F ;
                  x += byte1 ;

                  while ( byte1-- > 0 )
                  {
                     *lineStart = byte2 ;
                     lineStart += 4 ;
                  }
               }
               else
               {
                  *lineStart = byte1 ;
                  lineStart += 4 ;
                  x++ ;
               }
            }
         }
         else
         {
            // Alpha
            lineStart = outPos ;
            x = 0 ;
            for ( x = 0 ; x < width_ ; x++ )
            {
               *lineStart = 0xFF ;
               lineStart += 4 ;
            }
         }
      }

      outPos += outLineLength ;
   }

   return result ;
}

//---------------------------------------------------------------------------

// Returns data in ARGB format.
unsigned char* ReadPCXResource ( HMODULE hModule_, const char* resName_, unsigned long& width_, unsigned long& height_ )
{
   HRSRC resInfo = FindResource ( hModule_, resName_, "PCX" ) ;
   DWORD resLength = SizeofResource ( hModule_, resInfo ) ;
   HGLOBAL res = LoadResource ( hModule_, resInfo ) ;
   unsigned char* resData = (unsigned char*)LockResource ( res ) ;
   return ReadPCXMem ( resData, resLength, width_, height_ ) ;
}

//---------------------------------------------------------------------------

unsigned char* ReadPCXFile ( const char* fileName_, unsigned long& width_, unsigned long& height_ )
{
    unsigned char* result = NULL ;
    FILE* file ;

    file = fopen ( fileName_, "rb" ) ;

    if ( file != NULL )
    {
        unsigned char* data ;
        int size ;

        // Seek to the end of the file.
        fseek ( file, 0, SEEK_END ) ;

        // Find the current position, which is the size of the file as we're at the end.
        size = ftell ( file ) ;

        // Seek back to the start of the file.
        fseek ( file, 0, SEEK_SET ) ;

        // Create a buffer of the correct size.
        data = new unsigned char [ size ] ;

        // Read the whole file into the buffer.
        fread ( data, 1, size, file ) ;

        // Extra the image from the buffer.
        result = ReadPCXMem ( data, size, width_, height_ ) ;

        // Delete the buffer.
        delete [] data ;

        // Close the file.
        fclose ( file ) ;
    }

    return result ;
}

//---------------------------------------------------------------------------

